/*
	Parse.h
    Written by Saeed Amizadeh and Amir Nayyeri (PGen Project)

	Header classes for Parse.cpp
*/

#ifndef __PARSE_H
#define __PARSE_H

class CNode;
class CParseTable;

class CEdge
{
protected:
	STRING m_Input,m_Action;
	CNode *m_Head,*m_Tail;
	bool m_IsGraph;

public:
	friend class CGraph;
	int m_HeadID,m_TailID;
	
	CEdge(CNode*, CNode*, bool = false);
	CNode* GetHead() const;
	CNode* GetTail() const;
	STRING GetInput() const;
	STRING GetAction() const;
	bool IsGraph() const;

	void Load(istream&);
};
//---------------------------------------------------------------------------------------

class CNode
{
public:
	typedef std::vector<CEdge*> CEDGEVECTOR;
	friend class CGraph;
	
	CNode(int);
	~CNode();
	int GetID() const;
	bool IsFinal() const;
	CNode* JumpOverGraph(CCodeGenerator*) const;
	bool NextState(const STRING&, CNode*&, STRING&) const;
	bool NextGraph(CNode*&, const CParseTable*) const;
	void Load(istream&);

protected:
	int m_ID, m_EdgeNum;
	bool m_IsFinal;
	CEDGEVECTOR m_Edges;
};
//---------------------------------------------------------------------------------------

class CGraph
{
public:
	typedef std::vector<CNode*> CNODEVECTOR;
	
	CGraph();
	~CGraph();
	void Load(istream&);
	CNode* GetFirstNode() const;
	STRING GetName() const;

protected:
	int m_NodeNum;
	STRING m_Name;
	CNODEVECTOR m_Nodes;
};
//---------------------------------------------------------------------------------------

class CParseTable
{
public:
	typedef std::vector<CGraph*> CGRAPHVECTOR;

	CParseTable();
	~CParseTable();
	void Load(istream& Stream);
	CNode* GetStartNode() const;
	CNode* GetFirstNodeOf(const STRING&) const;

protected:
	int m_GraphNum;
	CGRAPHVECTOR m_Graphs;
};
//---------------------------------------------------------------------------------------

class CParser
{
public:
	typedef std::stack<CNode*> CNODESTACK;
	typedef std::stack<bool> BOOLSTACK;

	CParser(istream&, istream&, ostream&);
	~CParser();
	void Parse();
	CScanner* GetScanner() const;
	CCodeGenerator* GetCodeGenerator() const;

protected:
	CScanner* m_Scanner;
	CCodeGenerator* m_CodeGenerator;
	CParseTable* m_ParseTable;
	CNode* m_CurrentState;
};
//---------------------------------------------------------------------------------------

class CParseErrorException
{
public:
	STRING message;
	CParseErrorException(STRING message){this->message = message;}
};
//---------------------------------------------------------------------------------------

class CInvalidFormatException
{
public:
	STRING message;
	CInvalidFormatException(STRING message){this->message = message;}
};

#endif