/*
	scg.h
    Written by Saeed Amizadeh and Amir Nayyeri (PGen Project)
	
	Edited by Mohsen Lesani and Amir Moghimi
	Thanks to S.A. and A.N.

	Primitive header classes for Scanner and Code Generator
	You should develop these classes!
*/

#ifndef __SCG_H
#define __SCG_H

#include "FlexLexer.h"
#include "DSCP.h"
#include <stack>

typedef std::string STRING;

class Tree;
typedef std::vector<Tree *> ARGLIST;
std::string int2String(int);

// IntStack Class ///////////////////////////////////////////////////////////////////////
struct Node {
	int data;
	Node *next;
	Node(int d, Node *n) {data = d; next = n;}
};
class IntStack {
private:
	Node *first;

public:
	void push(int data) {first = new Node (data,first);};
	void push(int data[]) 
	{
         for (int i=1; data[i] != 0; i++)
         	first = new Node (data[i],first);
    }
	int pop() 
	{
      	int d = first->data; Node *f = first; first = first->next;
      	delete f; return d;
	}
	int top() { return first->data; }
};

//---------------------------------------------------------------------------------------
class CScanner : yyFlexLexer
{
protected:
	istream& inputStream;
	
	// Add your extra attributes here for Scanner

public:
	CScanner(istream&);
	STRING NextToken();
	STRING lexeme();

	// Add your extra methods here for Scanner
};
//---------------------------------------------------------------------------------------
class Except
{
public:
	Except(STRING message){this->message = message;}
	STRING message;
};

class FuncCell
{
public:
	FuncCell(BasicT retval, const STRING & name, ARGVECT * args){this->name = name; this->args = args; this->retval = retval;}

	int operator ==(const FuncCell & another){return ((name == another.name) && (*args == *(another.args)));}

	BasicT retval;
	STRING name;
	ARGVECT *args;
};

class SSCell
{
public:
	SSCell(const STRING & label){this->label = label;}
	SSCell(int column, int depth){columnnum = column; depthnum = depth;}
	SSCell(BasicT prevtype){this->prevtype = prevtype;}

	int columnnum, depthnum;
	STRING label;
	BasicT prevtype;
};

class Tree
{
public:
	Tree(STRING input, char type, ARGLIST *arguments = NULL)
	{
		data = input;
		left = right = NULL;
		this->type = type;
		this->arguments = arguments;
	}

	Tree(char input, char type, ARGLIST *arguments = NULL)
	{
		data += input;
		left = right = NULL;
		this->type = type;
		this->arguments = arguments;
	}

	~Tree();

	char type;
	STRING data;
	Tree *left, *right;
	ARGLIST *arguments;
};
//---------------------------------------------------------------------------------------
class CCodeGenerator
{
private:
	bool includePrintInt;
	int literalcounter, labelcounter;
	//Label providers
	Tree *root, *underconstruct, *temproot, *keep4later;
	//root -> each expression make it to point to the expression tree
	//underconstruct -> temp pointer for making the tree
	//temproot -> for Expressions that must kept for later use
	//keep4later -> for making delay between for Expressions to make code	

	ARGLIST *functionargs;
	//Temp for newing and adding to functions and arrays

	std::vector<DSCP *> myframe;
	//frame of function
	ARGVECT *myargs;
	//Passed argument types to the function
	int mybp;
	std::vector<FuncCell> funcprototypes;
	//Contain function prototypes

//////////////////////////////////////////////////////////////////////////
	int findDSCPinglobal( STRING what2find);
	int findwithdifferentargs( STRING what2find);
	int findDSCPinframe( STRING what2find);
	int searchforthisinfunctionframe( STRING what2find);

/////////////////////////////////////////////////////////////////////////
protected:
	STRING functionStream;
	STRING tempStream;
	STRING anotherStream;
	ostream& outputStream;

	std::stack<Tree *> TreeStk;
	std::stack<SSCell> SStk;
	std::vector<STRING> DataSeg;
	//The data segment that will made at the end of program
	std::vector<DSCP *> SymbolTable;
	// Add your extra attributes here for Code Generator (especially Semantic Stack)

public:
	CCodeGenerator(ostream&);
	void Generate(const STRING&, const STRING&, const STRING&);
	~CCodeGenerator();
	DSCP* findDSCP( STRING what2find);
	STRING nextLabel();
	STRING nextLiteral();


	// Add your extra methods here for Code Generator
};

#endif



